/*
 * MP3 Player, ATA support.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Contact: paul@pjrc.com
 *
 * File		: ata.h
 * Author	: MBA
 * Created	: nov, 2001
 *
 */

#ifndef _ATA_H_
#define _ATA_H_

// Defines for the ATA device configuration, as detected by the AtaQueryDevice function.
#define ATA_DEVICE_COUNT_MAX  2

#define ATA_DEVCONFIG_NONE    0
#define ATA_DEVCONFIG_UNKNOWN 1
#define ATA_DEVCONFIG_ATA     2
#define ATA_DEVCONFIG_ATAPI   3

extern unsigned char g_AtaDevConfig[ATA_DEVICE_COUNT_MAX];

// ATA register and register bit defines.
#define	memATA_BASEADDRESS	0xFF00
#define regATA_DATALSB		0x60	// data reg         in/out
#define regATA_DATAMSB		0x43	// data reg         in/out
#define regATA_DATA		0x60	// data reg         in/out
#define regATA_ERROR		0x62	// error            in    
#define regATA_FEATURE		0x62	// feature reg         out
#define regATA_SECCNT		0x64	// sector count     in/out
#define regATA_SECNR		0x66	// sector number    in/out
#define regATA_CYLLSB		0x68	// cylinder low     in/out
#define regATA_CYLMSB		0x6a	// cylinder high    in/out
#define regATA_DEVHEAD		0x6c	// device head      in/out
#define regATA_PRISTAT		0x6e	// primary status   in    
#define regATA_DEVCMD   	0x6e	// command             out
#define regATA_ALTSTAT		0x7c	// alternate status in    
#define regATA_DEVCTRL		0x7c	// device control      out
#define regATA_DEVADDR    	9   	// device address   in     

volatile xdata unsigned char at 0xFF60 ucregATA_DATALSB;
volatile xdata unsigned char at 0xFF43 ucregATA_DATAMSB;

// ATA error register (regATA_ERROR) bitmasks (msk)

#define mskATA_ER_ICRC		0x80    // ATA Ultra DMA bad CRC
#define mskATA_ER_BBK  		0x80    // ATA bad block
#define mskATA_ER_UNC  		0x40    // ATA uncorrected error
#define mskATA_ER_MC   		0x20    // ATA media change
#define mskATA_ER_IDNF 		0x10    // ATA id not found
#define mskATA_ER_MCR  		0x08    // ATA media change request
#define mskATA_ER_ABRT 		0x04    // ATA command aborted
#define mskATA_ER_NTK0 		0x02	// ATA track 0 not found
#define mskATA_ER_NDAM 		0x01    // ATA address mark not found

#define mskATAPI_ER_SNSKEY 	0xf0   // ATAPI sense key (mask)
#define mskATAPI_ER_MCR    	0x08   // ATAPI Media Change Request
#define mskATAPI_ER_ABRT   	0x04   // ATAPI command abort
#define mskATAPI_ER_EOM    	0x02   // ATAPI End of Media
#define mskATAPI_ER_ILI    	0x01   // ATAPI Illegal Length Indication

// ATAPI Interrupt Reason bits in the Sector Count reg (regATA_SECCNT)

#define regATA_SECCNT_P_TAG	0xf8   // ATAPI tag (mask)
#define regATA_SECCNT_P_REL	0x04   // ATAPI release
#define regATA_SECCNT_P_IO	0x02   // ATAPI I/O
#define regATA_SECCNT_P_CD	0x01   // ATAPI C/D

// bits 7-4 of the device/head (regATA_DEVHEAD) reg

#define regATA_DEVHEAD_DEV0 0xe0	//0xa0    // select device 0
#define regATA_DEVHEAD_DEV1 0xf0	//0xb0    // select device 1

// status reg (regATA_PRISTAT and regATA_ALTSTAT) bits

#define regATA_PRISTAT_BSY  0x80  // busy
#define regATA_PRISTAT_RDY  0x40  // ready
#define regATA_PRISTAT_DF   0x20  // device fault
#define regATA_PRISTAT_WFT  0x20  // write fault (old name)
#define regATA_PRISTAT_SKC  0x10  // seek complete
#define regATA_PRISTAT_SERV 0x10  // service
#define regATA_PRISTAT_DRQ  0x08  // data request
#define regATA_PRISTAT_CORR 0x04  // corrected
#define regATA_PRISTAT_IDX  0x02  // index
#define regATA_PRISTAT_ERR  0x01  // error

// digital output reg (regATA_DEVCTRL) bitmasks

#define regATA_DEVCTRL_HD15   0x00	// 0x08  // bit should always be set to one
#define regATA_DEVCTRL_SRST   0x04  // soft reset
#define regATA_DEVCTRL_NIEN   0x02  // disable interrupts

/**************************************************************/

// Most mandatory and optional ATA commands (from ATA-3),

#define cmdATA_CFA_ERASE_SECTORS            0xC0
#define cmdATA_CFA_REQUEST_EXT_ERR_CODE     0x03
#define cmdATA_CFA_TRANSLATE_SECTOR         0x87
#define cmdATA_CFA_WRITE_MULTIPLE_WO_ERASE  0xCD
#define cmdATA_CFA_WRITE_SECTORS_WO_ERASE   0x38
#define cmdATA_CHECK_POWER_MODE1            0xE5
#define cmdATA_CHECK_POWER_MODE2            0x98
#define cmdATA_DEVICE_RESET                 0x08
#define cmdATA_EXECUTE_DEVICE_DIAGNOSTIC    0x90
#define cmdATA_FLUSH_CACHE                  0xE7
#define cmdATA_FORMAT_TRACK                 0x50
#define cmdATA_ATANTIFY_DEVICE              0xEC
#define cmdATA_ATANTIFY_DEVICE_PACKET       0xA1
#define cmdATA_ATANTIFY_PACKET_DEVICE       0xA1
#define cmdATA_IDLE1                        0xE3
#define cmdATA_IDLE2                        0x97
#define cmdATA_IDLE_IMMEDIATE1              0xE1
#define cmdATA_IDLE_IMMEDIATE2              0x95
#define cmdATA_INITIALIZE_DRIVE_PARAMETERS  0x91
#define cmdATA_INITIALIZE_DEVICE_PARAMETERS 0x91
#define cmdATA_NOP                          0x00
#define cmdATA_PACKET                       0xA0
#define cmdATA_READ_BUFFER                  0xE4
#define cmdATA_READ_DMA                     0xC8
#define cmdATA_READ_DMA_QUEUED              0xC7
#define cmdATA_READ_MULTIPLE                0xC4
#define cmdATA_READ_SECTORS                 0x20
#define cmdATA_READ_VERIFY_SECTORS          0x40
#define cmdATA_RECALIBRATE                  0x10
#define cmdATA_SEEK                         0x70
#define cmdATA_SET_FEATURES                 0xEF
#define cmdATA_SET_MULTIPLE_MODE            0xC6
#define cmdATA_SLEEP1                       0xE6
#define cmdATA_SLEEP2                       0x99
#define cmdATA_STANDBY1                     0xE2
#define cmdATA_STANDBY2                     0x96
#define cmdATA_STANDBY_IMMEDIATE1           0xE0
#define cmdATA_STANDBY_IMMEDIATE2           0x94
#define cmdATA_WRITE_BUFFER                 0xE8
#define cmdATA_WRITE_DMA                    0xCA
#define cmdATA_WRITE_DMA_QUEUED             0xCC
#define cmdATA_WRITE_MULTIPLE               0xC5
#define cmdATA_WRITE_SECTORS                0x30
#define cmdATA_WRITE_VERIFY                 0x3C

void ide_write_register(unsigned char ideReg, unsigned char ideData);
unsigned char ide_read_register(unsigned char ideReg);
void ide_write_dataregisters(unsigned char dataMSB, unsigned char dataLSB);
unsigned int ide_read_dataregisters();

#pragma CALLEE-SAVES AtaReadRegister
extern unsigned char AtaReadRegister(unsigned char p_ucRegister);

#pragma CALLEE-SAVES AtaWriteRegister
extern void AtaWriteRegister(unsigned char p_ucRegister, unsigned char p_ucData);

extern int AtaQueryDevice(void);
extern int AtaResetAdapter(void);

#pragma CALLEE-SAVES AtaSelectDevice
char AtaSelectDevice(unsigned char p_ucDevice);

#endif

#if 1
#define STAT_ERR (ucStatus & regATA_PRISTAT_ERR)

#define HOST_BSY0_DRQ0 \
   while ( 1 ) \
   { \
      ucStatus = AtaReadRegister( regATA_PRISTAT ); \
/*      printf("ATA-HOST_BSY0_DRQ0 = %x\r\n", ucStatus); */\
      if ( ( ucStatus & (regATA_PRISTAT_ERR) ) != 0 ) \
         break; \
      if ( ( ucStatus & (regATA_PRISTAT_BSY/* | regATA_PRISTAT_DRQ*/) ) == 0 ) \
         break; \
   } \

#define HOST_BSY0 \
   while ( 1 ) \
   { \
      ucStatus = AtaReadRegister( regATA_ALTSTAT ); \
/*      printf("ATA-HOST_BSY0 = %x\r\n", ucStatus); */\
      if ( ( ucStatus & (regATA_PRISTAT_ERR) ) != 0 ) \
         break; \
      if ( ( ucStatus & (regATA_PRISTAT_BSY) ) == 0 ) \
         break; \
   } \

#define HOST_BSY0_DRQ1 \
   while ( 1 ) \
   { \
      ucStatus = AtaReadRegister( regATA_PRISTAT ); \
/*      printf("ATA-HOST_BSY0_DRQ1 = %x\r\n", ucStatus);*/ \
      if ( ( ucStatus & (regATA_PRISTAT_ERR) ) != 0 ) \
         break; \
      if ( ( ucStatus & (regATA_PRISTAT_BSY | regATA_PRISTAT_DRQ) ) == regATA_PRISTAT_DRQ ) \
         break; \
   } \

#define HOST_DRQ1 \
   while ( 1 ) \
   { \
      ucStatus = AtaReadRegister( regATA_PRISTAT ); \
/*      printf("ATA-HOST_DRQ1 = %x\r\n", ucStatus);*/\
      if ( ( ucStatus & (regATA_PRISTAT_ERR) ) != 0 ) \
         break; \
      if ( ( ucStatus & (regATA_PRISTAT_DRQ) ) != 0 ) \
         break; \
   } \

// #endif

#endif
