/*
 * MP3 Player, ATA support.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * Contact: paul@pjrc.com
 *
 * File		: ata.c
 * Author	: MBA
 * Created	: nov, 2001
 *
 */

#include "ata.h"
#include "printf.h"

#if 0
volatile xdata unsigned char at 0xFF60 reg_ide_data;
volatile xdata unsigned char at 0xFF43 reg_ide_buf_msb
#endif

// Global var holding the device configuration of the devices (max 2) attached to the IDE interface
// Used to determine whether to call the ATA or ATAPI "drivers" and/or helper functions.
unsigned char g_AtaDevConfig[ATA_DEVICE_COUNT_MAX];

/*********************
 * AtaReadRegister
 *	Wrapper (should become obsolete) to call the assembly function to read an 8-bit register
 *	given by the register argument
 *
 */
unsigned char AtaReadRegister(unsigned char p_ucRegister)
{
  return ide_read_register((unsigned char)p_ucRegister);
}

/*********************
 * AtaWriteRegister
 *	Wrapper (should become obsolete) to call the assembly function to write an 8-bit register
 *	given by the register argument and the 8-bit data to write.
 *
 */
void AtaWriteRegister(unsigned char p_ucRegister, unsigned char p_ucData)
{
  ide_write_register((unsigned char)p_ucRegister, p_ucData);
}

/*********************
 * AtaSelectDevice
 *	Selects the given device. Subsequent functions will address the selected device.
 *	The current implementation with 1 IDE interface should only accept device 0 and 1.
 *
 */
char AtaSelectDevice(unsigned char p_ucDevice)
{
  unsigned char ucStatus;

  HOST_BSY0_DRQ0;

  // Here we select the drive we really want to work with by
  // putting 0xA0 or 0xB0 in the Drive/Head register.

  AtaWriteRegister(regATA_DEVHEAD, p_ucDevice ? regATA_DEVHEAD_DEV1 : regATA_DEVHEAD_DEV0 );

  ucStatus = AtaReadRegister( regATA_ALTSTAT );

  // we always return 0. All errors are ignored.
  return 0;
}

/*********************
 * AtaQueryDevice
 *	Checks the host adapter and determines the number and type of drives attached.
 *	
 *	This function fills the global variable g_AtaDevConfig.
 *	For values see ata.h, the DEVCONFIG values.
 *
 *	The number of drives found is returned as the function result
 */
int AtaQueryDevice(void)
{
  int           iNumDev = 0;
  unsigned char ucSecCnt;
  unsigned char ucSecNr;
  unsigned char ucCylLow;
  unsigned char ucCylHigh;
  unsigned char ucStatus;
  int           iATA;

  iNumDev = 0;

  // Query the host adapter to determine the drives attached
  for (iATA = 0; iATA < ATA_DEVICE_COUNT_MAX; iATA++)
  {
    // Assume we have no device attached at all
    g_AtaDevConfig[iATA] = ATA_DEVCONFIG_NONE;
    
    // Issue a reset command for this (possible) device
    AtaResetAdapter();

    // Check the device again by reading the signature in the sectorcount/number registers.
    AtaWriteRegister(regATA_DEVHEAD, iATA == 0 ? regATA_DEVHEAD_DEV0 : regATA_DEVHEAD_DEV1);
    ucSecCnt = AtaReadRegister( regATA_SECCNT );
    ucSecNr  = AtaReadRegister( regATA_SECNR );
    
    if ((ucSecCnt == 0x01) && (ucSecNr == 0x01))
    {
      // At least something is out there, but still unknown...
      g_AtaDevConfig[iATA] = ATA_DEVCONFIG_UNKNOWN;
      
      // To determine ATA or ATAPI, read the signature in the cylinder registers
      ucCylLow  = AtaReadRegister( regATA_CYLLSB );
      ucCylHigh = AtaReadRegister( regATA_CYLMSB );
      ucStatus  = AtaReadRegister( regATA_PRISTAT );
      
      // Signature is ATAPI ???
      if ((ucCylLow == 0x14) && (ucCylHigh == 0xeb))
      {
        g_AtaDevConfig[iATA] = ATA_DEVCONFIG_ATAPI;
        printf("ATAPI\r\n");
        iNumDev++;
      }
      else
      {
        // Signature is ATA ???
        if ( ( ucCylLow == 0x00 ) && ( ucCylHigh == 0x00 ) && ( ucStatus != 0x00 ) )
        {
          g_AtaDevConfig[iATA] = ATA_DEVCONFIG_ATA;
          printf("ATA\r\n");
          iNumDev++;
        }
      }
    }
  }
  return iNumDev;
}

/*********************
 * AtaResetAdapter
 *	Resets the device(s) attached to the host adapter and automatically
 *	selects device 0.
 */

int AtaResetAdapter(void)
{
  unsigned char ucStatus;
  unsigned char ucDevCtrl;

  // setup register values
  ucDevCtrl = regATA_DEVCTRL_HD15 | regATA_DEVCTRL_NIEN;

  // Set and then reset the soft reset bit in the Device Control
  // register.  This causes device 0 be selected.

  AtaWriteRegister( regATA_DEVCTRL, ucDevCtrl | regATA_DEVCTRL_SRST);
  AtaWriteRegister( regATA_DEVCTRL, ucDevCtrl );
  
  while (1)
  {
    ucStatus = AtaReadRegister(regATA_PRISTAT);
    if ((ucStatus & (regATA_PRISTAT_BSY | regATA_PRISTAT_DRQ)) == 0)
      break;
  }
  AtaWriteRegister(regATA_DEVHEAD, regATA_DEVHEAD_DEV0);
  while (1)
  {
    ucStatus = AtaReadRegister(regATA_PRISTAT);
    if ((ucStatus & (regATA_PRISTAT_BSY | regATA_PRISTAT_DRQ)) == 0)
      break;
  }
  
  return 0;
}
